<?php
/**
 * Create the metabox
 * @link https://developer.wordpress.org/reference/functions/add_meta_box/
 */
function _event_create_metabox() {
	add_meta_box(
		'_event_metabox', // Metabox ID
		'Event info', // Title to display
		'_event_render_metabox', // Function to call that contains the metabox content
		'event', // Post type to display metabox on
		'normal', // Where to put it (normal = main colum, side = sidebar, etc.)
		'default' // Priority relative to other metaboxes
	);
}
add_action( 'add_meta_boxes', '_event_create_metabox' );

/**
 * Create the metabox default values
 * This allows us to save multiple values in an array, reducing the size of our database.
 * Setting defaults helps avoid "array key doesn't exit" issues.
 * @todo
 */
function _event_metabox_defaults() {
	return array(
		'start_date' => '',
		'start_time' => '',
		'end_date' => '',
		'end_time' => '',
		'address' => '',
	);
}
/**
 * Render the metabox markup
 * This is the function called in `_event_create_metabox()`
 */
function _event_render_metabox($post) {
	
	$saved = get_post_meta( $post->ID, '_event', true ); // Get the saved values
	$defaults = _event_metabox_defaults(); // Get the default values
	$details = wp_parse_args( $saved, $defaults ); // Merge the two in case any fields don't exist in the saved data
	?>
	<fieldset>
		<div class="admin_metabox_section">
			<label for="_event_custom_metabox_start_date">
				<?php
				esc_attr_e( 'Start Date', 'eduline-pro' );
				?>
			</label>
			<input type="text" name="_event_custom_metabox[start_date]" id="start_date" value="<?php echo esc_attr( $details['start_date'] ); ?>" autocomplete="off">
		</div>

		<div class="admin_metabox_section">
			<label for="_event_custom_metabox_start_time">
				<?php
				esc_attr_e( 'Start Time', 'eduline-pro' );
				?>
			</label>
			<p><?php echo esc_html__('Eg:- 10:00 am','eduline-pro');?></p>
			<input type="text" name="_event_custom_metabox[start_time]" id="start_time" value="<?php echo esc_attr( $details['start_time'] ); ?>">
		</div>

		<div class="admin_metabox_section">
			<label for="_event_custom_metabox_end_date">
				<?php
				esc_attr_e( 'End Date', 'eduline-pro' );
				?>
			</label>
			<input type="text" name="_event_custom_metabox[end_date]" id="end_date" value="<?php echo esc_attr( $details['end_date'] ); ?>" autocomplete="off">
		</div>

		<div class="admin_metabox_section">
			<label for="_event_custom_metabox_end_time">
				<?php
				esc_attr_e( 'End Time', 'eduline-pro' );
				?>
			</label>
			<p><?php echo esc_html__('Eg:- 5:00  pm','eduline-pro');?></p>
			<input type="text" name="_event_custom_metabox[end_time]" id="end_time"
			value="<?php echo esc_attr( $details['end_time'] ); ?>"
			>
		</div>

		<div class="admin_metabox_section">
			<label for="_event_custom_metabox_address">
				<?php
				esc_attr_e( 'Address', 'eduline-pro' );
				?>
			</label>
			<input type="text" name="_event_custom_metabox[address]" id="_event_custom_metabox_address" value="<?php echo esc_attr( $details['address'] ); ?>">
		</div>	
	</fieldset>

	<?php

	wp_nonce_field( '_event_form_metabox_nonce', '_event_form_metabox_process' );
}

//
// Save our data
//
/**
 * Save the metabox
 * @param  Number $post_id The post ID
 * @param  Array  $post    The post data
 */
function _event_save_metabox( $post_id, $post ) {
	// Verify that our security field exists. If not, bail.
	if ( !isset( $_POST['_event_form_metabox_process'] ) ) return;
	// Verify data came from edit/dashboard screen
	if ( !wp_verify_nonce( $_POST['_event_form_metabox_process'], '_event_form_metabox_nonce' ) ) {
		return $post->ID;
	}
	// Verify user has permission to edit post
	if ( !current_user_can( 'edit_post', $post->ID )) {
		return $post->ID;
	}
	// Check that our custom fields are being passed along
	// This is the `name` value array. We can grab all
	// of the fields and their values at once.
	if ( !isset( $_POST['_event_custom_metabox'] ) ) {
		return $post->ID;
	}
	/**
	 * Sanitize all data
	 * This keeps malicious code out of our database.
	 */
	// Set up an empty array
	$sanitized = array();
	// Loop through each of our fields
	foreach ( $_POST['_event_custom_metabox'] as $key => $detail ) {
		// Sanitize the data and push it to our new array
		// `wp_filter_post_kses` strips our dangerous server values
		// and allows through anything you can include a post.
		$sanitized[$key] = wp_filter_post_kses( $detail );
	}
	// Save our submissions to the database
	update_post_meta( $post->ID, '_event', $sanitized );
}
add_action( 'save_post', '_event_save_metabox', 1, 2 );
//
// Save a copy to our revision history
// This is optional, and potentially undesireable for certain data types.
// Restoring a a post to an old version will also update the metabox.
/**
 * Save events data to revisions
 * @param  Number $post_id The post ID
 */
function _event_save_revisions( $post_id ) {
	// Check if it's a revision
	$parent_id = wp_is_post_revision( $post_id );
	// If is revision
	if ( $parent_id ) {
		// Get the saved data
		$parent = get_post( $parent_id );
		$details = get_post_meta( $parent->ID, '_event', true );
		// If data exists and is an array, add to revision
		if ( !empty( $details ) && is_array( $details ) ) {
			// Get the defaults
			$defaults = _event_metabox_defaults();
			// For each default item
			foreach ( $defaults as $key => $value ) {
				// If there's a saved value for the field, save it to the version history
				if ( array_key_exists( $key, $details ) ) {
					add_metadata( 'post', $post_id, '_event_' . $key, $details[$key] );
				}
			}
		}
	}
}
add_action( 'save_post', '_event_save_revisions' );
/**
 * Restore events data with post revisions
 * @param  Number $post_id     The post ID
 * @param  Number $revision_id The revision ID
 */
function _event_restore_revisions( $post_id, $revision_id ) {
	// Variables
	$post = get_post( $post_id ); // The post
	$revision = get_post( $revision_id ); // The revision
	$defaults = _event_metabox_defaults(); // The default values
	$details = array(); // An empty array for our new metadata values
	// Update content
	// For each field
	foreach ( $defaults as $key => $value ) {
		// Get the revision history version
		$detail_revision = get_metadata( 'post', $revision->ID, '_event_' . $key, true );
		// If a historic version exists, add it to our new data
		if ( isset( $detail_revision ) ) {
			$details[$key] = $detail_revision;
		}
	}
	// Replace our saved data with the old version
	update_post_meta( $post_id, '_event', $details );
}
add_action( 'wp_restore_post_revision', '_event_restore_revisions', 10, 2 );
/**
 * Get the data to display on the revisions page
 * @param  Array $fields The fields
 * @return Array The fields
 */
function _event_get_revisions_fields( $fields ) {
	// Get our default values
	$defaults = _event_metabox_defaults();
	// For each field, use the key as the title
	foreach ( $defaults as $key => $value ) {
		$fields['_event_' . $key] = ucfirst( $key );
	}
	return $fields;
}
add_filter( '_wp_post_revision_fields', '_event_get_revisions_fields' );
/**
 * Display the data on the revisions page
 * @param  String|Array $value The field value
 * @param  Array        $field The field
 */
function _event_display_revisions_fields( $value, $field ) {
	global $revision;
	return get_metadata( 'post', $revision->ID, $field, true );
}
add_filter( '_wp_post_revision_field_my_meta', '_event_display_revisions_fields', 10, 2 );